==> communication-1.1.9-sources.jar: com/transsion/iot/communication/udp/UDPClient.kt <==
package com.transsion.iot.communication.udp

import android.util.Log
import com.google.gson.Gson
import com.google.gson.JsonSyntaxException
import com.transsion.iot.communication.packet.resp.BasicInfo
import java.io.IOException
import java.net.DatagramPacket
import java.net.DatagramSocket
import java.net.SocketException
import java.util.concurrent.CopyOnWriteArrayList
import java.util.concurrent.atomic.AtomicBoolean

/**
 * @Author fuhuang.fan
 * @Date 2022/7/19/019 15:05
 * @Description
 */
object UDPClient {
    private lateinit var mReceiveThread: SocketReceiveThread
    private var mPort = 6000
    private var mSocket: DatagramSocket? = null
    private var mMessageReceivers: CopyOnWriteArrayList<IUDPMessageReceiver> = CopyOnWriteArrayList()

    init {
        try {
            mSocket = DatagramSocket(mPort)
        } catch (e: SocketException) {
            e.printStackTrace()
        }
    }

    fun registerMessageReceiver(receiver: IUDPMessageReceiver) {
        mMessageReceivers.add(receiver)
    }

    fun unregisterMessageReceiver(receiver: IUDPMessageReceiver) {
        mMessageReceivers.remove(receiver)
    }

    private fun onReceiveMessage(basicInfo: BasicInfo?) {
        for (receiver in mMessageReceivers) {
            receiver.onMessageReceived(basicInfo)
        }
    }

    private fun onReceiveError(errMsg: String?) {
        for (receiver in mMessageReceivers) {
            receiver.onErrorReceived(errMsg)
        }
    }

    fun startReceive() {
        try {
            mReceiveThread = SocketReceiveThread()
            mReceiveThread.start()
        } catch (e: Exception) {
            onReceiveError(e.message)
        }
    }

    fun stopReceive() {
        mReceiveThread.threadExit()
    }

    class SocketReceiveThread : Thread() {
        private var threadExit = AtomicBoolean(false)
        override fun run() {
            val buffer = ByteArray(1024)
            while (!threadExit.get()) {
                try {
                    val datagramPacket = DatagramPacket(buffer, buffer.size)
                    mSocket?.receive(datagramPacket)
                    val receiveData = String(buffer, 0, datagramPacket.length)
                    Log.d("UDPClient", "receiveData=$receiveData")
                    val basicInfo = Gson().fromJson(receiveData, BasicInfo::class.java)
                    Log.d("UDPClient", "basicInfo=$basicInfo")
                    onReceiveMessage(basicInfo)
                } catch (e: IOException) {
                    e.printStackTrace()
                    onReceiveError(e.message)
                    threadExit()
                } catch (e: JsonSyntaxException) {
                    e.printStackTrace()
                    onReceiveError(e.message)
                    threadExit()
                }
            }
        }

        fun threadExit() {
            threadExit.set(true)
        }
    }
}

==> communication-1.2.0-sources.jar: com/transsion/iot/communication/udp/UDPClient.kt <==
package com.transsion.iot.communication.udp

import android.util.Log
import com.google.gson.Gson
import com.google.gson.JsonSyntaxException
import com.transsion.iot.communication.packet.resp.BasicInfo
import java.io.IOException
import java.net.DatagramPacket
import java.net.DatagramSocket
import java.net.InetSocketAddress
import java.net.SocketException
import java.util.concurrent.CopyOnWriteArrayList
import java.util.concurrent.atomic.AtomicBoolean

/**
 * @Author fuhuang.fan
 * @Date 2022/7/19/019 15:05
 * @Description
 */
object UDPClient {
    private lateinit var mReceiveThread: SocketReceiveThread
    private var mPort = 6000
    private var mSocket: DatagramSocket? = null
    private var mMessageReceivers: CopyOnWriteArrayList<IUDPMessageReceiver> = CopyOnWriteArrayList()

    init {
        try {
            mSocket = DatagramSocket(mPort)
        } catch (e: SocketException) {
            e.printStackTrace()
        }
    }

    fun registerMessageReceiver(receiver: IUDPMessageReceiver) {
        mMessageReceivers.add(receiver)
    }

    fun unregisterMessageReceiver(receiver: IUDPMessageReceiver) {
        mMessageReceivers.remove(receiver)
    }

    private fun onReceiveMessage(basicInfo: BasicInfo?) {
        for (receiver in mMessageReceivers) {
            receiver.onMessageReceived(basicInfo)
        }
    }

    private fun onReceiveError(errMsg: String?) {
        for (receiver in mMessageReceivers) {
            receiver.onErrorReceived(errMsg)
        }
    }

    fun startReceive() {
        try {
            if (mSocket == null) {
                mSocket = DatagramSocket(null)
                mSocket?.reuseAddress = true
                mSocket?.bind(InetSocketAddress(mPort))
            }
            mReceiveThread = SocketReceiveThread()
            mReceiveThread.start()
        } catch (e: Exception) {
            onReceiveError(e.message)
        }
    }

    fun stopReceive() {
        mReceiveThread.threadExit()
    }

    class SocketReceiveThread : Thread() {
        private var threadExit = AtomicBoolean(false)
        override fun run() {
            val buffer = ByteArray(1024)
            while (!threadExit.get()) {
                try {
                    val datagramPacket = DatagramPacket(buffer, buffer.size)
                    mSocket?.receive(datagramPacket)
                    val receiveData = String(buffer, 0, datagramPacket.length)
                    Log.d("UDPClient", "receiveData=$receiveData")
                    val basicInfo = Gson().fromJson(receiveData, BasicInfo::class.java)
                    Log.d("UDPClient", "basicInfo=$basicInfo")
                    onReceiveMessage(basicInfo)
                } catch (e: IOException) {
                    e.printStackTrace()
                    onReceiveError(e.message)
                    threadExit()
                } catch (e: JsonSyntaxException) {
                    e.printStackTrace()
                    onReceiveError(e.message)
                    threadExit()
                }
            }
        }

        fun threadExit() {
            threadExit.set(true)
        }
    }
}
